#!/bin/bash

set -o errexit

trap

NAG_OK=0
NAG_WARN=1
NAG_CRIT=2
NAG_UNKNOWN=3

FAILED_UNITS_STR=""
NUM_FAILED=0
STATUS_STR=OK
RET_VAL=$NAG_OK
REMOTE_HOST='localhost'
BOOT_TIME_WARN_SEC=300
BOOT_TIME_CRIT_SEC=600
BOOT_TIME_STR=""

USE_SSH=""

trap "echo 'UNKNOWN - '; exit $NAG_UNKNOWN" EXIT

print_usage(){
cat <<EOF
This script checks status of systemd and reports boot time as well as failed services.
 Usage: check_systemd -H hostname -c TIME -w TIME
-H [USER@]HOSTNAME      Connection information, where HOSTNAME can be an IP adress too
-w T                    If boot took longer than T seconds (no decimal notation allowed), status is reported as WARNING.
-c T                    If boot took longer than T seconds (no decimal notation allowed), status is reported as CRITICAL.
-s                      Use ssh to connect instead of -H option from systemd as -H can trigger 'Failed to send message: Transport endpoint is not connected' in some cases.
EOF
}


while getopts "H:w:c:s" opt; do
	case $opt in 
		H) 
			REMOTE_HOST=$OPTARG
			;;
		w)
			BOOT_TIME_WARN_SEC=$OPTARG
			;;
		c)
			BOOT_TIME_CRIT_SEC=$OPTARG
			;;
		s)
			USE_SSH="y"
			;;
		?)
			echo "Invalid options $opt"
			print_usage
			exit $NAG_UNKNOWN
			;;
	esac
done

if [ -n "${USE_SSH}" ]; then
	SYSTEMD_ANALYZE_BIN="ssh ${REMOTE_HOST} systemd-analyze"
	SYSTEMCTL_BIN="ssh ${REMOTE_HOST} systemctl"
else
	SYSTEMD_ANALYZE_BIN="systemd-analyze -H ${REMOTE_HOST}"
	SYSTEMCTL_BIN="systemctl -H ${REMOTE_HOST}"
fi

ANALYZE_STR=$($SYSTEMD_ANALYZE_BIN time --no-pager | head -n 1)

# convert boot time to seconds
BOOT_TIME_SEC=$(echo ${ANALYZE_STR} | sed -e 's/^.*= //' -e 's/min//' -e 's/s//' | awk '{print (NF>2?$(NF-2)*3600:0)+(NF>1?$(NF-1)*60:0)+$(NF)}')

# round toward zero since float compares are hard in bash
BOOT_TIME_SEC_ROUND=$(echo $BOOT_TIME_SEC | sed -e 's/\..*$//' -e 's/,.*$//')

# compare boot time to limits
if [ $BOOT_TIME_SEC_ROUND -gt $BOOT_TIME_WARN_SEC ]; then
	BOOT_TIME_STR="Boot time exceeded ${BOOT_TIME_WARN_SEC}s, "
	STATUS_STR=WARNING
	RET_VAL=$NAG_WARN
fi

if [ $BOOT_TIME_SEC_ROUND -gt $BOOT_TIME_CRIT_SEC ]; then
	BOOT_TIME_STR="Boot time exceeded ${BOOT_TIME_WARN_SEC}s, "
	STATUS_STR=CRITICAL
	RET_VAL=$NAG_CRIT
fi
	
# get list of failed units
LST_FAILED_UNITS=$($SYSTEMCTL_BIN list-units --failed --full --no-legend --no-pager | awk '{print $1}')
#>sudo -u nagios systemctl -H debian-xmpp.private.lan list-units --failed --full --no-legend --no-pager
#block-badips-onboot.service loaded failed failed Applies iptables rules to block known IPs doing nasty stuff"
#block-badips-timer.service  loaded failed failed Timer service for running block badips script periodically 

# assemble list of failed units as human readable string
if [ ! -z "$LST_FAILED_UNITS" ]; then
	NUM_FAILED=$(printf '%s\n' $LST_FAILED_UNITS | wc -l)
	STATUS_STR=CRITICAL
	FAILED_UNITS_STR="Failed units: $LST_FAILED_UNITS, "
	RET_VAL=$NAG_CRIT
fi

# get total number of units
NUM_UNITS=$($SYSTEMCTL_BIN list-units --no-pager --no-legend | wc -l)

# assemble output string
# sample output of check_systemd (python reference implementation) 
# |count_units=312 startup_time=175.857;60;120 units_activating=0 units_active=219 units_failed=0 units_inactive=93
echo $STATUS_STR - $FAILED_UNITS_STR$BOOT_TIME_STR$ANALYZE_STR$PERF_DATA"|count_units=${NUM_UNITS} startup_time=${BOOT_TIME_SEC} units_failed=${NUM_FAILED}"

trap "" EXIT
exit $RET_VAL

